import java.io.*;
import ssdbms.ssd.*;
import ssdbms.manager.primitives.*;
import storage.primitiveStorage.*;
import storage.intermediateStorage.*;
import storage.advancedStorage.*;
import storage.*;
import DbStorage.*;
import DbStorage.driver.*;
import DbStorage.primitives.*;
import DbStorage.manager.*;
import concurrency.*;
import kernelInterface.*;
import java.util.*;

public class DbTools
{
    static public void createDb(File dbDirectory, int pageSize, int bufferSize, int replaceAlgorithm)
	throws Exception
    {
	DbStorageManager dbsm=new DbStorageManager(new PrimitiveStorageManager(pageSize,bufferSize,replaceAlgorithm));
	UnitsStorageManager usm=dbsm.getUnitsStorageManager();
	File storageCfg=new File(dbDirectory,"storage.cfg");
	dbsm.createDefaultSettings(dbDirectory);
	DbUnitAddress header=new DbUnitAddress();
	usm.findFreeUnit(DbStorageManager.DICTIONARY_SEGMENT,header);
	usm.addUnit(DbStorageManager.DICTIONARY_SEGMENT,header);

	usm.writeUnit(DbStorageManager.DICTIONARY_SEGMENT,header.getSpaceNumber(),header.getUnitNumber(),
		      DictionaryHeaderDriver.IDS_ROOT_OFFSET,DbUnitAddress.SIZE,DbUnitAddress.NULL.getContent(),0);
	usm.writeUnit(DbStorageManager.DICTIONARY_SEGMENT,header.getSpaceNumber(),header.getUnitNumber(),
		      DictionaryHeaderDriver.IDS_HEAD_OFFSET,DbUnitAddress.SIZE,DbUnitAddress.NULL.getContent(),0);
	usm.writeUnit(DbStorageManager.DICTIONARY_SEGMENT,header.getSpaceNumber(),header.getUnitNumber(),
		      DictionaryHeaderDriver.NAMES_ROOT_OFFSET,DbUnitAddress.SIZE,DbUnitAddress.NULL.getContent(),0);
	usm.writeUnit(DbStorageManager.DICTIONARY_SEGMENT,header.getSpaceNumber(),header.getUnitNumber(),
		      DictionaryHeaderDriver.NAMES_HEAD_OFFSET,DbUnitAddress.SIZE,DbUnitAddress.NULL.getContent(),0);


	File dbCfg=new File(dbDirectory,"db.cfg");
	Properties dbConfig=new Properties();
	dbConfig.setProperty("pageSize",String.valueOf(pageSize));
	dbConfig.setProperty("bufferSize",String.valueOf(bufferSize));
	dbConfig.setProperty("dictionaryHeaderSpace",String.valueOf(header.getSpaceNumber()));
	dbConfig.setProperty("dictionaryHeaderUnit",String.valueOf(header.getUnitNumber()));
	dbConfig.setProperty("replaceAlgorthm",String.valueOf(replaceAlgorithm));
	dbConfig.store(new FileOutputStream(dbCfg),"database configuration");
	usm.getAdvancedStorageManager().synchronize();
    }

    static public void createDb(File dbDirectory, int pageSize, int bufferSize)
	throws Exception
    {
	createDb(dbDirectory,pageSize,bufferSize,PrimitiveStorageManager.LRU_ALGORITHM);
    }

    static public void createDb(File dbDirectory)
	throws Exception
    {
	createDb(dbDirectory,4096,4*1024*1024);
    }

    static public DbInterface loadDb(File dbDirectory)
	throws Exception
    {
	File dbCfg=new File(dbDirectory,"db.cfg");
	Properties dbConfig=new Properties();
	dbConfig.load(new FileInputStream(dbCfg));
	int pageSize=Integer.parseInt(dbConfig.getProperty("pageSize"));
	int bufferSize=Integer.parseInt(dbConfig.getProperty("bufferSize"));
	int headerSpace=Integer.parseInt(dbConfig.getProperty("dictionaryHeaderSpace"));
	long headerUnit=Long.parseLong(dbConfig.getProperty("dictionaryHeaderUnit"));
	int replaceAlgorithm=Integer.parseInt(dbConfig.getProperty("replaceAlgorthm"));
	
	DbStorageManager dbsm=new DbStorageManager(new PrimitiveStorageManager(pageSize,bufferSize,replaceAlgorithm));
	File storageCfg=new File(dbDirectory,"storage.cfg");
	dbsm.getStorageToolsManager().loadSettingsFromFile(storageCfg);
	DbUnitAddress header=new DbUnitAddress(headerSpace,headerUnit);
	DbManager dbm=new DbManager(dbsm,header);
	ConcurrencyManager cm=new ConcurrencyManager(0);
	return new DbInterface(dbm,cm);
    }

}



