import java.io.*;
import ssdbms.ssd.*;
import ssdbms.manager.primitives.*;
import storage.primitiveStorage.*;
import storage.intermediateStorage.*;
import storage.advancedStorage.*;
import storage.*;
import DbStorage.*;
import DbStorage.driver.*;
import DbStorage.primitives.*;
import DbStorage.manager.*;
import concurrency.*;
import kernelInterface.*;
import java.util.*;

public class Pru03
{
    DbInterface dbi;
    DbManager dbm;

    public Pru03()
	throws Exception
    {
	int pageSize=1024;

	DbStorageManager dbsm=new DbStorageManager(new PrimitiveStorageManager(pageSize,4*1024*1024));
	dbsm.createDefaultSettings(new File("."));
	DbUnitAddress header=new DbUnitAddress(0,0);
	DictionaryHeaderDriver.createDictionaryHeader(dbsm,0,header);
	DictionaryManager.formatHeader(dbsm,0,header);
	dbm=new DbManager(dbsm,header);
	ConcurrencyManager cm=new ConcurrencyManager(0);
	dbi=new DbInterface(dbm,cm);
    }

    public class Proccess extends Thread
    {
	public void showID(ID id)
	    throws Exception
	{
	    System.out.print(id.toString()+" ");
	    int type=dbi.getType(id);
	    
	    if (type==SSDValue.INTEGER)
		System.out.print(((IntegerValue)(dbi.getContent(id))).getValue());
	    else if (type==SSDValue.REAL)
		System.out.print(((RealValue)(dbi.getContent(id))).getValue());
	    else if (type==SSDValue.STRING)
		System.out.print("\""+((StringValue)(dbi.getContent(id))).getValue()+"\"");
	    else if (type==SSDValue.BYTE_SEQUENCE)
		{
		    byte[] b=((ByteSequenceValue)(dbi.getContent(id))).getValue();
		    System.out.print("[ ");
		    for (int i=0; i<b.length; i++)
			System.out.print(Integer.toHexString(0xFF&b[i])+" ");
		    System.out.print("]");
		}
	    else if (type==SSDValue.BLOB)
		System.out.print("BLOB("+((BLOBManager)(dbi.getContent(id))).length()+")");
	    else if (type==SSDValue.CLOB)
		System.out.print("CLOB("+((CLOBManager)(dbi.getContent(id))).length()+")");
	    else if (type==SSDValue.NON_PRIMITIVE)
		{
		    System.out.print("{ ");
		    SetOfLabeledSSD s=((SetOfLabeledSSD)(dbi.getContent(id)));
		    while (s.hasNext())
			{
			    LabeledSSD lssd=s.next();
			    System.out.print(lssd.getLabel().toString()+":");
			    showID(lssd.getID());
			    if (s.hasNext())
				System.out.print(",");
			    System.out.print(" ");
			}
		    System.out.print("}");
		}
	    else
		System.out.println("UNKNOWN_CONTENT");
	}

	public void showSSDTable(Label name)
	    throws Exception
	{
	    System.out.print(name.toString()+":");
	    showID(dbi.getRoot(name));
	    System.out.println("");
	}
	
    }

    public class Proccess1 extends Proccess
    {
	public void run()
	{
	    try {
		System.out.println("P1: BEGINNING TRANSACTION");
		ID id00=dbi.createNonPrimitive();

		System.out.println("P1: creating ssd-table ...");
		dbi.addSSDTable(new Label("t1"),id00);
		System.out.println("P1: ssd-table created ...");

		ID id01=dbi.createInteger(new IntegerValue(1));
		ID id02=dbi.createReal(new RealValue(3.1416));
		ID id03=dbi.createString(new StringValue("hello world..."));
		byte[] b={(byte)0xFe, (byte)0xFF, (byte)0x00, (byte)0x01, (byte)0x02};
		ID id04=dbi.createByteSequence(new ByteSequenceValue(b));
		ID id05=dbi.createNonPrimitive();
		ID id06=dbi.createBLOB();
		ID id07=dbi.createCLOB();
		
		dbi.add(id00,new Label("number"),id01);
		dbi.add(id00,new Label("number"),id02);
		dbi.add(id00,new Label("string"),id03);
		dbi.add(id00,new Label("bytes"),id04);
		dbi.add(id00,new Label("lobs"),id05);
		dbi.add(id05,new Label("lob"),id06);
		dbi.add(id05,new Label("lob"),id07);
		
		System.out.println("P1: ENDING TRANSACTION");
		dbi.commit();
		//dbm.getAdvancedStorageManager().synchronize();
	    } catch (Exception e) {
		System.err.println("ERROR: "+e.getMessage());
		e.printStackTrace();
	    }
	}
    }
	
    public class Proccess2 extends Proccess
    {
	public void run()
	{
	    try {
		sleep(1000);
		System.out.println("P2: BEGINNING TRANSACTION");
		ID id00=dbi.createNonPrimitive();
		
		System.out.println("P2: accessing ssd-table ...");
		showSSDTable(new Label("t1"));
		System.out.println("P2: ssd-table accessed ...");
		
		System.out.println("P2: ENDING TRANSACTION");
		
		//dbm.getAdvancedStorageManager().synchronize();
	    } catch (Exception e) {
		System.err.println("ERROR: "+e.getMessage());
		e.printStackTrace();
	    }
	}
    }

    public class Proccess3 extends Proccess
    {
	public void run()
	{
	    try {
		sleep(1000);
		System.out.println("P3: BEGINNING TRANSACTION");
		ID id00=dbi.createNonPrimitive();
		
		System.out.println("P3: accessing and modifing ssd-table ...");
		dbi.add(dbi.getRoot(new Label("t1")),new Label("X"),dbi.createString(new StringValue("X")));

		showSSDTable(new Label("t1"));
		System.out.println("P3: ssd-table accessed ...");
		
		System.out.println("P3: ENDING TRANSACTION");
		
		//dbm.getAdvancedStorageManager().synchronize();
	    } catch (Exception e) {
		System.err.println("ERROR: "+e.getMessage());
		e.printStackTrace();
	    }
	}
    }

    public void go()
	throws Exception
    {
	(new Proccess1()).start();
	(new Proccess2()).start();
	(new Proccess3()).start();
    }

    static public void main(String[] argv)
    {
	try {
	    (new Pru03()).go();
	} catch (Exception e) {
	    System.err.println("ERROR: "+e.getMessage());
	    e.printStackTrace();
	}
    }
}
