package DbStorage;

/**
 * This class is used to convert some data types to their representation as a 
 * byte array or viceversa.
 */
public class DataConvertion
{
    /**
     * Converts a <code>long</code> integer to a byte array.
     */
    static public byte[] longToBytes(long value)
    {
	byte[] dataAux=new byte[8];
	for (int j=0; j<8; j++)
	    dataAux[j]=(byte)(0xFF & (value>>(8*j)));
	return dataAux;
    }

    /**
     * Converts a byte array to a <code>long</code> integer.
     */
    static public long bytesToLong(byte[] value)
    {
	long l=0l;
	for (int j=0; j<value.length && j<8; j++)
	    l |= ((0xFFl & value[j])<<(j*8));
	return l;
    }

    /**
     * Converts a <code>double</code> floating point number to a byte array.
     */
    static public byte[] doubleToBytes(double value)
    {
	long l=Double.doubleToRawLongBits(value);
	return longToBytes(l);
    }

    /**
     * Converts a <code>double</code> floating point number to a byte array.
     */
    static public double bytesToDouble(byte[] value)
    {
	return Double.longBitsToDouble(bytesToLong(value));
    }

    /**
     * Converts a <code>String</code> to a byte array.
     */
    static public byte[] stringToBytes(String value)
    {
	int l=value.length();
	byte[] dataAux=new byte[l*2];
	for (int i=0; i<l; i++)
	    {
		dataAux[i*2]=(byte)(0xFF & value.charAt(i));
		dataAux[i*2+1]=(byte)(0xFF & (value.charAt(i)>>8));
	    }
	return dataAux;
    }

    /**
     * Converts a <code>String</code> to a byte array.
     */
    static public byte[] stringToBytes(String value, int off, int len)
    {
	int l=value.length()-off;
	if (l>len)
	    l=len;
	byte[] dataAux=new byte[l*2];
	for (int i=0; i<l; i++)
	    {
		dataAux[i*2]=(byte)(0xFF & value.charAt(i+off));
		dataAux[i*2+1]=(byte)(0xFF & (value.charAt(i+off)>>8));
	    }
	return dataAux;
    }

    /**
     * Converts a byte array to a <code>String</code>.
     */
    static public String bytesToString(byte[] value)
    {
	int l=value.length/2;
	char[] dataAux=new char[l];
	for (int i=0; i<l; i++)
	    dataAux[i] = (char)((0xFF & value[i*2]) | ((0xFF & value[i*2+1])<<8)); 
	return new String(dataAux);
    }

}
