package DbStorage;

import ssdbms.ssd.ID;

/**
 * This class represents an identifier for semistructured data,
 * this identifier is actually a <code>DbUnitAddress</code>.
 */
public class PhysicalID extends DbUnitAddress implements ID
{
    /** Represents a NULL (invalid) PhsicalID */
    static public final PhysicalID NULL=new PhysicalID();

    /** 
     * Checks if this physical identifier is equals to other one.
     * @param id The physical identifier to compare.
     */
    public boolean equals(ID id)
    {
	if (id instanceof PhysicalID)
	    return equals((DbUnitAddress)id);
	else 
	    return false;
    } 

    /**
     * Checks if this identifier is a Not valid Identifier.
     * @return <code>true</code> If this identifier is invalid, <code>false</code> otherwise.
     */
    public boolean isNID()
    {
	return isNull();
    }

    /** 
     * Compares two physical identifiers in a partial order.
     * @param id The physical identifier to compare.
     * @return <code>1</code> if this one if greater, <code>0</code> if is equal 
     *         or <code>1-</code> if is less.
     */
    public int compareTo(PhysicalID id)
    {
	if (isNID() && id.isNID())
	    return 0;
	else if (isNID())
	    return -1;
	else if (id.isNID())
	    return 1;
	else if (getSpaceNumber()==id.getSpaceNumber())
	    {
		if (getUnitNumber()==id.getUnitNumber())
		    return 0;
		else if (getUnitNumber()>id.getUnitNumber())
		    return 1;
		else
		    return -1;
	    }
	else if (getSpaceNumber()>id.getSpaceNumber())
	    return 1;
	else
	    return -1;
    }

    /**
     * Returns the string representation of the ID.
     */
    public String toString()
    {
	return ""+getSpaceNumber()+"-"+getUnitNumber();
    }
}
