package DbStorage.driver;

import DbStorage.*;
import storage.*;
import java.io.*;

/**
 * This class drives the page of addresses in the LOB segment.
 */
public final class LobAddressesUnitDriver
{
    /**
     * Creates a addresses' unit.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param newUnitAddress The address of the created structure in the LOB segment.
     */
    static public void createAddressesUnit(DbStorageManager dbsm, long transactionNumber,
					   DbUnitAddress newUnitAddress)
	throws StorageException, IOException
    {
	dbsm.addUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,newUnitAddress);
	if (!newUnitAddress.isNull())
	    {
		long d=dbsm.LOB_SEGMENT_UNIT_SIZE/DbUnitAddress.SIZE;
		for (long i=0; i<d; i++)
		    setAddress(dbsm,transactionNumber,newUnitAddress,(int)i,DbUnitAddress.NULL);
	    }
    }

    /**
     * Gets the address in the specified position of the page in the LOB segment.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param unitAddress The address of the page that contains the addresses descriptor.
     * @param position The position of the address.
     * @param destiny Where the retrieved unit address will be stored.
     */
    static public void getAddress(DbStorageManager dbsm, long transactionNumber,
				  DbUnitAddress unitAddress, int position, DbUnitAddress destiny)
	throws StorageException, IOException
    {
	dbsm.readDbUnitAddress(transactionNumber,DbStorageManager.LOB_SEGMENT,
			       unitAddress,position*DbUnitAddress.SIZE,destiny);
    }

    /**
     * Sets the address in the specified position of the page in the LOB segment.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param unitAddress The address of the page that contains the addresses descriptor.
     * @param position The position of the address.
     * @param source The unit to set.
     */
    static public void setAddress(DbStorageManager dbsm, long transactionNumber,
				  DbUnitAddress unitAddress, int position, DbUnitAddress source)
	throws StorageException, IOException
    {
	dbsm.writeDbUnitAddress(transactionNumber,DbStorageManager.LOB_SEGMENT,
				unitAddress,position*DbUnitAddress.SIZE,source);
    }
}
