package DbStorage.driver;

import DbStorage.*;
import storage.*;
import java.io.*;

/**
 * This class drives the pages of content in the LOB segment.
 */
public final class LobContentUnitDriver extends SsdDescriptorDriver
{
    /**
     * Creates a content unit.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param newUnitAddress The address of the created unit in the LOB segment.
     */
    static public void createContentUnit(DbStorageManager dbsm, long transactionNumber,
					 DbUnitAddress newUnitAddress)
	throws StorageException, IOException
    {
	dbsm.addUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,newUnitAddress);
    }

    /**
     * Gets the part of datum's content stored in the specified page.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param unitAddress The address of the page.
     * @param position The position in the page from which the reading starts.
     * @param maxLength The maximun number of bytes to be got.
     * @param b Where the content will be stored.
     * @param off The begining position of b where the content will be stored.
     * @return The number of bytes recovered bytes.
     */
    static public int getContent(DbStorageManager dbsm, long transactionNumber,
				 DbUnitAddress unitAddress, int position, int maxLength, byte[] b, int off)
	throws StorageException, IOException
    {
	int l=dbsm.LOB_SEGMENT_UNIT_SIZE-position;
	if (l>maxLength)
	    l=maxLength;
	dbsm.readUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,unitAddress,position,l,b,off);
	return l;
    }
    
    /**
     * Sets the part of datum's content stored in the specified page.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param unitAddress The address of the SSD descriptor.
     * @param position The position in the page from which the writting starts.
     * @param maxLength The maximun number of bytes to be setted.
     * @param b Where the content is stored.
     * @param off The begining position of b where the content is stored.
     * @return The number of wroten bytes.
     */
    static public int setContent(DbStorageManager dbsm, long transactionNumber,
			  DbUnitAddress unitAddress, int position, int maxLength, byte[] b, int off)
	throws StorageException, IOException
    {
	int l=dbsm.LOB_SEGMENT_UNIT_SIZE-position;
	if (l>maxLength)
	    l=maxLength;
	dbsm.writeUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,unitAddress,position,l,b,off);
	return l;
    }
}
