package DbStorage.manager;

import DbStorage.*;
import DbStorage.driver.*;
import storage.*;
import java.io.*;

/**
 * This class implements the metods to storage BLOB objects.
 */
public final class BlobManager
{
    /**
     * Changes the length of the BLOB datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to resize.
     * @param newLength The new length of the datum.
     */
    static public void setLength(DbStorageManager dbsm, long transactionNumber,
				 PhysicalID id, long newLength)
	throws StorageException, IOException, OutOfLOBException 
    {
	LobManager.setSize(dbsm,transactionNumber,id,newLength);
    }

    /**
     * Returns the length of the datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to resize.
     * @return The length of the datum.
     */
    static public long getLength(DbStorageManager dbsm, long transactionNumber, PhysicalID id)
    throws StorageException, IOException
    {
	return LobPrimitiveSsdDescriptorDriver.getContentSize(dbsm,transactionNumber,id);
    }

    /**
     * Writes a byte array into a BLOB datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to wrote.
     * @param from The position of the BLOB from which the witting starts.
     * @param maxLength The maximum number of bytes to write.
     * @param b The array contains the bytes to write.
     * @param off The offset of <code>b</code> from which the bytes are retrieved.
     * @return The number of written bytes. 
     */
    static public int write(DbStorageManager dbsm, long transactionNumber,
			    PhysicalID id, long from, int maxLength, byte[] b, int off)
	throws StorageException, IOException, OutOfLOBException
    {
	return LobManager.write(dbsm,transactionNumber,id,from,maxLength,b,off);
    }

    /**
     * Writes a byte array into a BLOB datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to wrote.
     * @param from The position of the BLOB from which the witting starts.
     * @param b The array contains the bytes to write.
     * @return The number of written bytes. 
     */
    static public int write(DbStorageManager dbsm, long transactionNumber,
			    PhysicalID id, long from, byte[] b)
	throws StorageException, IOException, OutOfLOBException
    {
	return write(dbsm,transactionNumber,id,from,b.length,b,0);
    }
 

    /**
     * Reads from a BLOB datum and stores the readed in a byte array.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to read.
     * @param from The position of the BLOB from which the reading starts.
     * @param maxLength The maximum number of bytes to read.
     * @param b The array where the readed bytes will be stored.
     * @param off The offset of <code>b</code> from which the readed bytes are stored.
     * @return The number of readed bytes. 
     */
    static public int read(DbStorageManager dbsm, long transactionNumber,
			   PhysicalID id, long from, int maxLength, byte[] b, int off)
	throws StorageException, IOException, OutOfLOBException
    {
	return LobManager.read(dbsm,transactionNumber,id,from,maxLength,b,off);
    }

    /**
     * Reads from a BLOB datum and stores the readed in a byte array.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to read.
     * @param from The position of the BLOB from which the reading starts.
     * @param b The array where the readed bytes will be stored.
     * @return The number of readed bytes. 
     */
    static public int read(DbStorageManager dbsm, long transactionNumber,
			   PhysicalID id, long from, byte[] b)
	throws StorageException, IOException, OutOfLOBException
    {
	return read(dbsm,transactionNumber,id,from,b.length,b,0);
    }
}
