package DbStorage.manager;

import DbStorage.*;
import DbStorage.driver.*;
import storage.*;
import java.io.*;

/**
 * This class implements the metods to storage CLOB objects.
 */
public final class ClobManager
{
    /**
     * Changes the length of the CLOB datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to resize.
     * @param newLength The new length of the datum (in characters).
     */
    static public void setLength(DbStorageManager dbsm, long transactionNumber,
				 PhysicalID id, long newLength)
	throws StorageException, IOException, OutOfLOBException 
    {
	LobManager.setSize(dbsm,transactionNumber,id,newLength*2);
    }

    /**
     * Returns the length of the CLOB datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to resize.
     * @return The length of the datum (in characters).
     */
    static public long getLength(DbStorageManager dbsm, long transactionNumber, PhysicalID id)
    throws StorageException, IOException
    {
	return LobPrimitiveSsdDescriptorDriver.getContentSize(dbsm,transactionNumber,id)/2;
    }

    /**
     * Writes a byte array into a CLOB datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to wrote.
     * @param from The position of the CLOB from which the witting starts.
     * @param maxLength The maximum number of characters to write.
     * @param str The string contains the characters to write.
     * @param off The offset of <code>str</code> from which the characters are retrieved.
     * @return The number of written characters. 
     */
    static public int write(DbStorageManager dbsm, long transactionNumber,
			    PhysicalID id, long from, int maxLength, String str, int off)
	throws StorageException, IOException, OutOfLOBException
    {
	byte[] b=DataConvertion.stringToBytes(str,off,maxLength);
	return LobManager.write(dbsm,transactionNumber,id,from*2,b.length,b,0)/2;
    }

    /**
     * Writes a byte array into a CLOB datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to wrote.
     * @param from The position of the CLOB from which the witting starts.
     * @param str The string contains the characters to write.
     * @return The number of written characters. 
     */
    static public int write(DbStorageManager dbsm, long transactionNumber,
			    PhysicalID id, long from, String str)
	throws StorageException, IOException, OutOfLOBException
    {
	byte[] b=DataConvertion.stringToBytes(str);
	return LobManager.write(dbsm,transactionNumber,id,from*2,b.length,b,0)/2;
    }
    

    /**
     * Reads from a CLOB datum and stores the readed in a string.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to read.
     * @param from The position of the CLOB from which the reading starts.
     * @param maxLength The maximum number of characters to read.
     * @return A string contains the readed characters. 
     */
    static public String read(DbStorageManager dbsm, long transactionNumber,
			      PhysicalID id, long from, int maxLength)
	throws StorageException, IOException, OutOfLOBException
    {
	long l=getLength(dbsm,transactionNumber,id)-from;
	if (l>maxLength)
	    l=maxLength;
	byte[] b=new byte[((int)l)*2];
	long rb=LobManager.read(dbsm,transactionNumber,id,from*2,((int)l)*2,b,0);
	return DataConvertion.bytesToString(b);
    }
}
