package DbStorage.manager;

import DbStorage.*;
import DbStorage.driver.*;
import ssdbms.ssd.LabeledSSD;
import ssdbms.ssd.Label;
import ssdbms.ssd.SSDException;

/**
 * This class represents a pair formed of a <code>PhysicalID</code> and a string name.
 */
public final class DictionaryEntry
{
    private String name;
    private PhysicalID id;

    /**
     * Constructs <code>DictionaryEntry</code> with null components.
     */
    public DictionaryEntry()
    {
	this.name=null;
	this.id=null;
    }

    /**
     * Constructs <code>DictionaryEntry</code> with specified components.
     */
    public DictionaryEntry(String name, PhysicalID id)
    {
	this.name=name;
	this.id=id;
    }

    /**
     * Returns the name component.
     */
    public String getName()
    {
	return name;
    }

    /**
     * Returns the ID component.
     */
    public PhysicalID getID()
    {
	return id;
    }

    /**
     * Sets the name component.
     */
    public void setName(String name)
    {
	this.name=name;
    }

    /**
     * Sets the ID component.
     */
    public void setID(PhysicalID id)
    {
	this.id=id;
    }

    /**
     * This class is used to compare two <code>DictionaryEntry</code>s
     * using their ID component first and their name component second. 
     */
    static public class ByIDComparator
    {
	/**
	 * Compares two <code>DictionaryEntry</code>s
	 * using their ID component first and their name component second. 
	 */
	static public int compare(DictionaryEntry ls1, DictionaryEntry ls2)
	{
	    int r1=ls1.id.compareTo(ls2.id);
	    if (r1==0)
		{
		    int r2=ls1.name.compareTo(ls2.name);
		    if (r2<0)
			return -1;
		    else if (r2>0)
			return 1;
		    else
			return 0;
		}
	    else if (r1<0)
		return -1;
	    else
		return 1;
	}
    }

    /**
     * This class is used to compare two <code>DictionaryEntry</code>s
     * using their name component first and their ID component second. 
     */
    static public class ByNameComparator
    {
	/**
	 * Compares two <code>DictionaryEntry</code>s
	 * using their name component first and their ID component second. 
	 */
	static public int compare(DictionaryEntry ls1, DictionaryEntry ls2)
	{
	    int r1=ls1.name.compareTo(ls2.name);
	    if (r1==0)
		{
		    int r2=ls1.id.compareTo(ls2.id);
		    if (r2<0)
			return -1;
		    else if (r2>0)
			return 1;
		    else
			return 0;
		}
	    else if (r1<0)
		return -1;
	    else
		return 1;
	}
    }
}
