package DbStorage.manager;

import DbStorage.*;
import DbStorage.driver.*;
import storage.*;
import java.io.*;

/**
 * This class implements the metods to storage LOB objects.
 */
public final class LobManager
{
    static public final long MAX_LOB_SIZE=0x7FFFFFFFFFFFFFFFl;


    // Extension methods.

    static private long extendInIndirectUnit(DbStorageManager dbsm, long transactionNumber,
					    DbUnitAddress indirectUnit, long startUnit, long numUnits)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long start=startUnit;
	long len=d-startUnit;
	if (len>numUnits)
	    len=numUnits;
	long r=0;
	DbUnitAddress dataUnit=new DbUnitAddress();
	while (len>0)
	    {
		LobContentUnitDriver.createContentUnit(dbsm,transactionNumber,dataUnit);
		LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indirectUnit,(int)start,dataUnit);
		start++;
		r++;
		len--;
	    }
	return r;
    }


    static private long extendInDoubleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
						  DbUnitAddress doubleIndirectUnit, long startUnit, long numUnits)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long indirect=startUnit/d;
	long start=startUnit;
	long len=d*d-startUnit;
	if (len>numUnits)
	    len=numUnits;
	long l=0;
	long r=0;
	DbUnitAddress indirectUnit=new DbUnitAddress();
	while (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,doubleIndirectUnit,
						  (int)indirect,indirectUnit);
		if (indirectUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,indirectUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,doubleIndirectUnit,
							  (int)indirect,indirectUnit);
		    }
		l=extendInIndirectUnit(dbsm,transactionNumber,indirectUnit,start%d,len);
		start+=l;
		len-=l;
		r+=l;
		indirect++;
	    }
	return r;
    }


    static private long extendInTripleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
						 DbUnitAddress tripleIndirectUnit, long startUnit, long numUnits)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long doubleIndirect=startUnit/(d*d);
	long start=startUnit;
	long len=d*d*d-startUnit;
	if (len>numUnits)
	    len=numUnits;
	long l=0;
	long r=0;
	DbUnitAddress doubleIndirectUnit=new DbUnitAddress();
	while (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,tripleIndirectUnit,
						  (int)doubleIndirect,doubleIndirectUnit);
		if (doubleIndirectUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,doubleIndirectUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,tripleIndirectUnit,
							  (int)doubleIndirect,doubleIndirectUnit);
		    }
		l=extendInDoubleIndirectUnit(dbsm,transactionNumber,doubleIndirectUnit,start%(d*d),len);
		start+=l;
		len-=l;
		r+=l;
		doubleIndirect++;
	    }
	return r;
    }

    static private long extendInIndexUnit(DbStorageManager dbsm, long transactionNumber,
					 DbUnitAddress indexUnit, long startUnit, long numUnits)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;
	long lim1=d-4;
	long lim2=lim1+d;
	long lim3=lim2+d*d;
	long lim4=lim3+d*d*d;

	DbUnitAddress auxUnit=new DbUnitAddress();

	long start=startUnit;
	long len=lim4-startUnit;
	if (len>numUnits)
	    len=numUnits;
	int r=0;
	if (len>0 && start<lim1)
	    for (long i=0; len>0 && i<lim1; i++)
		if (start==i)
		    {
			LobContentUnitDriver.createContentUnit(dbsm,transactionNumber,auxUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,
							  (int)start,auxUnit);
			start++;
			len--;
			r++;
		    }
    	long l=0;
	if (len>0 && start<lim2)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-4),auxUnit);
		if (auxUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,auxUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,
							  (int)(d-4),auxUnit);
		    }
		l=extendInIndirectUnit(dbsm,transactionNumber,auxUnit,start-lim1,len);
		start+=l;
		len-=l;
		r+=l;
	    }
	if (len>0 && start<lim3)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-3),auxUnit);
		if (auxUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,auxUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,
							  (int)(d-3),auxUnit);
		    }
		l=extendInDoubleIndirectUnit(dbsm,transactionNumber,auxUnit,start-lim2,len);
		start+=l;
		len-=l;
		r+=l;
	    }
	if (len>0 && start<lim4)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-2),auxUnit);
		if (auxUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,auxUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,
							  (int)(d-2),auxUnit);
		    }
		l=extendInTripleIndirectUnit(dbsm,transactionNumber,auxUnit,start-lim3,len);
		start+=l;
		len-=l;
		r+=l;
	    }
	return r;
    }
    
    static private void extendInDatum(DbStorageManager dbsm, long transactionNumber,
				      PhysicalID id, long startUnit, long numUnits)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long start=startUnit;
	long len=numUnits;
	long lim=0;

	boolean first=true;
	DbUnitAddress indexUnit=new DbUnitAddress();
	while (len>0)
	    {
		long prevLim=lim;
		lim+=(d-4)*p+d*p+d*d*p+d*d*d*p;
		
		if (start<lim)
		    {
			if (first)
			    {
				LobPrimitiveSsdDescriptorDriver.getContentExtension(dbsm,transactionNumber,id,
										    indexUnit);
				if (indexUnit.isNull())
				    {
					LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,indexUnit);
					LobPrimitiveSsdDescriptorDriver.setContentExtension(dbsm,transactionNumber,
											    id,indexUnit);
				    }
				first=false;
			    }
			else
			    {
				DbUnitAddress auxUnit=new DbUnitAddress();
				auxUnit.copyFrom(indexUnit);
				LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,auxUnit,(int)(d-1),indexUnit);
				if (indexUnit.isNull())
				    {
					LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,indexUnit);
					LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,
									  auxUnit,(int)(d-1),indexUnit);
				    }
			    }
			long l=extendInIndexUnit(dbsm,transactionNumber,indexUnit,start-prevLim,len);
			start+=l;
			len-=l;
		    }
		else
		    {
			if (first)
			    {
				LobPrimitiveSsdDescriptorDriver.getContentExtension(dbsm,transactionNumber,id,
										    indexUnit);
				first=false;
			    }
			else
			    LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-1),indexUnit);
		    }
	    }
    }

    static private void extend(DbStorageManager dbsm, long transactionNumber,
			      PhysicalID id, long newSize)
	throws StorageException, IOException 
    {
	long lim=dbsm.ID_SEGMENT_UNIT_SIZE-LobPrimitiveSsdDescriptorDriver.CONTENT_OFFSET;
	
	if (newSize>lim)
	    {
		long p=dbsm.getPageSize();
		long d=p/DbUnitAddress.SIZE;
		long size=LobPrimitiveSsdDescriptorDriver.getContentSize(dbsm,transactionNumber,id);
		long lobSize=size<=lim?0:(size-lim);
		long newLobSize=newSize<=lim?0:(newSize-lim);
		long lobUnits=lobSize==0?0:(1+(lobSize-1)/p);
		long newLobUnits=newLobSize==0?0:(1+(newLobSize-1)/p);
		long newUnits=newLobUnits-lobUnits;
		if (newUnits>0)
		    extendInDatum(dbsm,transactionNumber,id,lobUnits,newLobUnits-lobUnits);
	    }
	LobPrimitiveSsdDescriptorDriver.setContentSize(dbsm,transactionNumber,id,newSize);
    }


    // Tuncation methods.

    static private void truncateInIndirectUnit(DbStorageManager dbsm, long transactionNumber,
					      DbUnitAddress indirectUnit, long startUnit)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long start=startUnit;
	DbUnitAddress dataUnit=new DbUnitAddress();
	while (start<d)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indirectUnit,(int)start,dataUnit);
		if (!dataUnit.isNull())
		    {
		    	LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indirectUnit,
							  (int)start,DbUnitAddress.NULL);
			dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,dataUnit);
		    }
		start++;
	    }
    }


    static private void truncateInDoubleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
						    DbUnitAddress doubleIndirectUnit, long startUnit)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long indirect=startUnit/d;
	DbUnitAddress indirectUnit=new DbUnitAddress();
	if (indirect<d)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,doubleIndirectUnit,
						  (int)indirect,indirectUnit);
		if (!indirectUnit.isNull())
		    {
			truncateInIndirectUnit(dbsm,transactionNumber,indirectUnit,startUnit%d);
			if (startUnit%d==0)
			    {
				LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,doubleIndirectUnit,
								  (int)indirect,DbUnitAddress.NULL);
				dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,indirectUnit);
			    }
		    }
		indirect++;
	    }
	while (indirect<d)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,doubleIndirectUnit,
						  (int)indirect,indirectUnit);
		if (!indirectUnit.isNull())
		    {
			truncateInIndirectUnit(dbsm,transactionNumber,indirectUnit,0);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,doubleIndirectUnit,
							  (int)indirect,DbUnitAddress.NULL);
			dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,indirectUnit);
		    }
		indirect++;
	    }
    }

    static private void truncateInTripleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
						    DbUnitAddress tripleIndirectUnit, long startUnit)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long doubleIndirect=startUnit/(d*d);
	DbUnitAddress doubleIndirectUnit=new DbUnitAddress();
	if (doubleIndirect<d)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,tripleIndirectUnit,
						  (int)doubleIndirect,doubleIndirectUnit);
		if (!doubleIndirectUnit.isNull())
		    {
			truncateInDoubleIndirectUnit(dbsm,transactionNumber,doubleIndirectUnit,startUnit%(d*d));
			if (startUnit%(d*d)==0)
			    {
				LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,tripleIndirectUnit,
								  (int)doubleIndirect,DbUnitAddress.NULL);
				dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,doubleIndirectUnit);
			    }
		    }
		doubleIndirect++;
	    }
	while (doubleIndirect<d)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,tripleIndirectUnit,
						  (int)doubleIndirect,doubleIndirectUnit);
		if (!doubleIndirectUnit.isNull())
		    {
			truncateInDoubleIndirectUnit(dbsm,transactionNumber,doubleIndirectUnit,0);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,tripleIndirectUnit,
							  (int)doubleIndirect,DbUnitAddress.NULL);
			dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,doubleIndirectUnit);
		    }
		doubleIndirect++;
	    }
    }


    static private void truncateInIndexUnit(DbStorageManager dbsm, long transactionNumber,
					 DbUnitAddress indexUnit, long startUnit)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;
	long lim1=d-4;
	long lim2=lim1+d;
	long lim3=lim2+d*d;
	long lim4=lim3+d*d*d;

	DbUnitAddress auxUnit=new DbUnitAddress();

	if (startUnit<lim1)
	    for (long i=0; i<lim1; i++)
		if (startUnit<=i)
		    {
			LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)i,auxUnit);
			if (!auxUnit.isNull())
			    {
				LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)i,
								  DbUnitAddress.NULL);
				dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,auxUnit);
			    }
		    }
	if (startUnit<lim2)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-4),auxUnit);
		if (!auxUnit.isNull())
		    {
			if (startUnit<=lim1)
			    {
				truncateInIndirectUnit(dbsm,transactionNumber,auxUnit,0);
				LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)(d-4),
								  DbUnitAddress.NULL);
				dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,auxUnit);
			    }
		    	else
			    truncateInIndirectUnit(dbsm,transactionNumber,auxUnit,startUnit-lim1);
		    }
	    }
	if (startUnit<lim3)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-3),auxUnit);
		if (!auxUnit.isNull())
		    {
			if (startUnit<=lim2)
			    {
				truncateInDoubleIndirectUnit(dbsm,transactionNumber,auxUnit,0);
				LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)(d-3),
								  DbUnitAddress.NULL);
				dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,auxUnit);
			    }
		    	else
			    truncateInIndirectUnit(dbsm,transactionNumber,auxUnit,startUnit-lim2);
		    }
	    }
	if (startUnit<lim4)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-2),auxUnit);
		if (!auxUnit.isNull())
		    {
			if (startUnit<=lim3)
			    {
				truncateInDoubleIndirectUnit(dbsm,transactionNumber,auxUnit,0);
				LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)(d-2),
								  DbUnitAddress.NULL);
				dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,auxUnit);
			    }
		    	else
			    truncateInIndirectUnit(dbsm,transactionNumber,auxUnit,startUnit-lim3);
		    }
	    }
    }


    static private void truncateInDatum(DbStorageManager dbsm, long transactionNumber,
				PhysicalID id, long startUnit)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long lim=0;
	DbUnitAddress indexUnit=new DbUnitAddress();
	DbUnitAddress prevIndexUnit=new DbUnitAddress();
	prevIndexUnit.setNull();
	DbUnitAddress auxUnit=new DbUnitAddress();
	LobPrimitiveSsdDescriptorDriver.getContentExtension(dbsm,transactionNumber,id,indexUnit);
	while (!indexUnit.isNull())
	    {
		long prevLim=lim;
		lim+=(d-4)*p+d*p+d*d*p+d*d*d*p;
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-1),auxUnit);

		if (startUnit<lim)
		    {
			if (startUnit<=prevLim)
			    {
				if (startUnit==0)
				    LobPrimitiveSsdDescriptorDriver.setContentExtension(dbsm,transactionNumber,
											id,DbUnitAddress.NULL);
				else if (!prevIndexUnit.isNull())
				    LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,prevIndexUnit,
								      (int)(d-1),DbUnitAddress.NULL);
				prevIndexUnit.copyFrom(DbUnitAddress.NULL);
				truncateInIndexUnit(dbsm,transactionNumber,indexUnit,0);
				dbsm.dropUnit(transactionNumber,DbStorageManager.LOB_SEGMENT,indexUnit);
			    }
			else
			    {
				prevIndexUnit.copyFrom(indexUnit);
				truncateInIndexUnit(dbsm,transactionNumber,indexUnit,startUnit-prevLim);
			    }
		    }
		else
		    prevIndexUnit.copyFrom(indexUnit);
		indexUnit.copyFrom(auxUnit);
	    }
    }
  
    static private void truncate(DbStorageManager dbsm, long transactionNumber,
				PhysicalID id, long newSize)
	throws StorageException, IOException 
    {
	long lim=dbsm.ID_SEGMENT_UNIT_SIZE-LobPrimitiveSsdDescriptorDriver.CONTENT_OFFSET;
	
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;
	long size=LobPrimitiveSsdDescriptorDriver.getContentSize(dbsm,transactionNumber,id);
	long lobSize=size<=lim?0:(size-lim);
	long newLobSize=newSize<=lim?0:(newSize-lim);
	long lobUnits=lobSize==0?0:(1+(lobSize-1)/p);
	long newLobUnits=newLobSize==0?0:(1+(newLobSize-1)/p);
	long removeUnits=lobUnits-newLobUnits;
	if (removeUnits>0)
	    truncateInDatum(dbsm,transactionNumber,id,newLobUnits);
	
	LobPrimitiveSsdDescriptorDriver.setContentSize(dbsm,transactionNumber,id,newSize);
    }

    /**
     * Changes the size of the LOB datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to resize.
     * @param newSize The new size of the datum.
     */
    static public void setSize(DbStorageManager dbsm, long transactionNumber,
			       PhysicalID id, long newSize)
	throws StorageException, IOException, OutOfLOBException 
    {
	if (newSize<0)
	    throw new OutOfLOBException();
	long size=LobPrimitiveSsdDescriptorDriver.getContentSize(dbsm,transactionNumber,id);
	if (newSize>size)
	    extend(dbsm,transactionNumber,id,newSize);
	else if (newSize<size)
	    truncate(dbsm,transactionNumber,id,newSize);
    }

    
    // Filling methods.

    static private void fillIndirectUnit(DbStorageManager dbsm, long transactionNumber,
				       DbUnitAddress indirectUnit)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;
	DbUnitAddress dataUnit=new DbUnitAddress();
	for (long i=0; i<d; i++)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indirectUnit,(int)i,dataUnit);
		if (dataUnit.isNull())
		    {
			LobContentUnitDriver.createContentUnit(dbsm,transactionNumber,dataUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indirectUnit,(int)i,dataUnit);
		    }
	    }
	
    }

    static private void fillDoubleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
					     DbUnitAddress doubleIndirectUnit)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;
	DbUnitAddress indirectUnit=new DbUnitAddress();
	for (long i=0; i<d; i++)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,doubleIndirectUnit,(int)i,indirectUnit);
		if (indirectUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,indirectUnit);
			fillIndirectUnit(dbsm,transactionNumber,indirectUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,doubleIndirectUnit,
							  (int)i,indirectUnit);
		    }
		else
		    fillIndirectUnit(dbsm,transactionNumber,indirectUnit);
	    }
	
    }

    static private void fillTripleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
					     DbUnitAddress tripleIndirectUnit)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;
	DbUnitAddress doubleIndirectUnit=new DbUnitAddress();
	for (long i=0; i<d; i++)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,tripleIndirectUnit,
						  (int)i,doubleIndirectUnit);
		if (doubleIndirectUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,doubleIndirectUnit);
			fillDoubleIndirectUnit(dbsm,transactionNumber,doubleIndirectUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,tripleIndirectUnit,
							  (int)i,doubleIndirectUnit);
		    }
		else
		    fillDoubleIndirectUnit(dbsm,transactionNumber,doubleIndirectUnit);
	    }
	
    }

    static private void fillIndexUnit(DbStorageManager dbsm, long transactionNumber,
				     DbUnitAddress indexUnit)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;
	DbUnitAddress auxUnit=new DbUnitAddress();
	for (long i=0; i<(d-4); i++)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)i,auxUnit);
		if (auxUnit.isNull())
		    {
			LobContentUnitDriver.createContentUnit(dbsm,transactionNumber,auxUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)i,auxUnit);
		    }
	    }

	LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-4),auxUnit);
	if (auxUnit.isNull())
	    {
		LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,auxUnit);
		fillIndirectUnit(dbsm,transactionNumber,auxUnit);
		LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)(d-4),auxUnit);
	    }
	else
	    fillIndirectUnit(dbsm,transactionNumber,auxUnit);

	LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-3),auxUnit);
	if (auxUnit.isNull())
	    {
		LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,auxUnit);
		fillDoubleIndirectUnit(dbsm,transactionNumber,auxUnit);
		LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)(d-3),auxUnit);
	    }
	else
	    fillDoubleIndirectUnit(dbsm,transactionNumber,auxUnit);

	LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-2),auxUnit);
	if (auxUnit.isNull())
	    {
		LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,auxUnit);
		fillTripleIndirectUnit(dbsm,transactionNumber,auxUnit);
		LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)(d-2),auxUnit);
	    }
	else
	    fillTripleIndirectUnit(dbsm,transactionNumber,auxUnit);	
    }

    
    // Writting methods.

    static private int writeInDataUnit(DbStorageManager dbsm, long transactionNumber,
				      DbUnitAddress dataUnit, long from, int length, byte[] b, int off)
	throws StorageException, IOException
    {
	return LobContentUnitDriver.setContent(dbsm,transactionNumber,dataUnit,(int)from,length,b,off);
    }

    static private int writeInIndirectUnit(DbStorageManager dbsm, long transactionNumber,
					  DbUnitAddress indirectUnit, long from, int length, byte[] b, int off)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long addressNumber=0;
	DbUnitAddress dataUnit=new DbUnitAddress();
	while ((addressNumber+1)*p-1<from)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indirectUnit,
						  (int)addressNumber,dataUnit);
		if (dataUnit.isNull())
		    {
			LobContentUnitDriver.createContentUnit(dbsm,transactionNumber,dataUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indirectUnit,
							  (int)addressNumber,dataUnit);
		    }
		addressNumber++;
	    }
	long start=from;
	int offset=off;
	long len=p*d-from;
	if (len>length)
	    len=length;
	int l=0;
	int r=0;
	while (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indirectUnit,
						  (int)addressNumber,dataUnit);
		if (dataUnit.isNull())
		    {
			LobContentUnitDriver.createContentUnit(dbsm,transactionNumber,dataUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indirectUnit,
							  (int)addressNumber,dataUnit);
		    }
		l=writeInDataUnit(dbsm,transactionNumber,dataUnit,start%p,(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
		addressNumber++;
	    }
	return r;
    }

    
    static private int writeInDoubleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
						DbUnitAddress doubleIndirectUnit, long from, int length, 
						byte[] b, int off)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long addressNumber=0;
	DbUnitAddress indirectUnit=new DbUnitAddress();
	while ((addressNumber+1)*p*d-1<from)
	    {
		DbUnitAddress dataUnit=new DbUnitAddress();
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,doubleIndirectUnit,
						  (int)addressNumber,indirectUnit);
		if (indirectUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,indirectUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,doubleIndirectUnit,
							  (int)addressNumber,indirectUnit);
		    }
		fillIndirectUnit(dbsm,transactionNumber,indirectUnit);
		addressNumber++;
	    }
	long start=from;
	int offset=off;
	long len=p*d*d-from;
	if (len>length)
	    len=length;
	int l=0;
	int r=0;
	while (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,doubleIndirectUnit,
						  (int)addressNumber,indirectUnit);
		if (indirectUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,indirectUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,doubleIndirectUnit,
							  (int)addressNumber,indirectUnit);
		    }
		l=writeInIndirectUnit(dbsm,transactionNumber,indirectUnit,start%(p*d),(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
		addressNumber++;
	    }
	return r;
    }

    
    static private int writeInTripleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
						DbUnitAddress tripleIndirectUnit, long from, int length, 
						byte[] b, int off)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long addressNumber=0;
	DbUnitAddress doubleIndirectUnit=new DbUnitAddress();
	while ((addressNumber+1)*p*d*d-1<from)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,tripleIndirectUnit,
						  (int)addressNumber,doubleIndirectUnit);
		if (doubleIndirectUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,doubleIndirectUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,tripleIndirectUnit,
							  (int)addressNumber,doubleIndirectUnit);
		    }
		fillDoubleIndirectUnit(dbsm,transactionNumber,doubleIndirectUnit);
		addressNumber++;
	    }
	long start=from;
	int offset=off;
	long len=p*d*d*d-from;
	if (len>length)
	    len=length;
	int l=0;
	int r=0;
	while (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,tripleIndirectUnit,
						  (int)addressNumber,doubleIndirectUnit);
		if (doubleIndirectUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,doubleIndirectUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,tripleIndirectUnit,
							  (int)addressNumber,doubleIndirectUnit);
		    }
		l=writeInDoubleIndirectUnit(dbsm,transactionNumber,doubleIndirectUnit,
					    start%(p*d*d),(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
		addressNumber++;
	    }
	return r;
    }

    
    static private int writeInIndexUnit(DbStorageManager dbsm, long transactionNumber,
					DbUnitAddress indexUnit, long from, int length, byte[] b, int off)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;
	long lim1=(d-4)*p;
	long lim2=lim1+d*p;
	long lim3=lim2+d*d*p;
	long lim4=lim3+d*d*d*p;

	DbUnitAddress auxUnit=new DbUnitAddress();

	long start=from;
	int offset=off;
	long len=lim4-from;
	if (len>length)
	    len=length;
	int l=0;
	int r=0;
	for (long i=0; len>0 && i<d-4; i++)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)i,auxUnit);
		if (auxUnit.isNull())
		    {
			LobContentUnitDriver.createContentUnit(dbsm,transactionNumber,auxUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)i,auxUnit);
		    }
		if (start>=i*p && start<(i+1)*p)
		    {
			l=writeInDataUnit(dbsm,transactionNumber,auxUnit,start%p,(int)len,b,offset);
			start+=l;
			offset+=l;
			len-=l;
			r+=l;
		    }
	    }
	if (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-4),auxUnit);
		if (auxUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,auxUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)(d-4),auxUnit);
		    }
		if (start<lim2)
		    {
			l=writeInIndirectUnit(dbsm,transactionNumber,auxUnit,start-lim1,(int)len,b,offset);
			start+=l;
			offset+=l;
			len-=l;
			r+=l;
		    }
		else
		    fillIndirectUnit(dbsm,transactionNumber,auxUnit);
	    }
	if (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-3),auxUnit);
		if (auxUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,auxUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)(d-3),auxUnit);
		    }
		if (start<lim3)
		    {
			l=writeInDoubleIndirectUnit(dbsm,transactionNumber,auxUnit,start-lim2,(int)len,b,offset);
			start+=l;
			offset+=l;
			len-=l;
			r+=l;
		    }
		else
		    fillDoubleIndirectUnit(dbsm,transactionNumber,auxUnit);
	    }
	if (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-2),auxUnit);
		if (auxUnit.isNull())
		    {
			LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,auxUnit);
			LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,indexUnit,(int)(d-2),auxUnit);
		    }
		l=writeInTripleIndirectUnit(dbsm,transactionNumber,auxUnit,start-lim3,(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
	    }
	return r;
    }
    
    /**
     * Writes a byte array into a LOB datum.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to wrote.
     * @param from The position of the LOB from which the witting starts.
     * @param maxLength The maximum number of bytes to write.
     * @param b The array contains the bytes to write.
     * @param off The offset of <code>b</code> from which the bytes are retrieved.
     * @return The number of written bytes. 
     */
    static public int write(DbStorageManager dbsm, long transactionNumber,
			    PhysicalID id, long from, int maxLength, byte[] b, int off)
	throws StorageException, IOException, OutOfLOBException
    {
	if (from<0 || !(from+maxLength>=0 && from+maxLength<=MAX_LOB_SIZE))
	    throw new OutOfLOBException();

	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long size=LobPrimitiveSsdDescriptorDriver.getContentSize(dbsm,transactionNumber,id);

	long start=from;
	int offset=off;
	long len=maxLength<0?0:maxLength;
	int l=0;
	int r=0;

	long lim=dbsm.ID_SEGMENT_UNIT_SIZE-LobPrimitiveSsdDescriptorDriver.CONTENT_OFFSET;

	if (start<lim)
	    {
		l=LobPrimitiveSsdDescriptorDriver.setContent(dbsm,transactionNumber,id,(int)start,(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
	    }
	boolean first=true;
	long prevLim=0;
	while (len>0)
	    {
		prevLim=lim;
		lim+=(d-4)*p+d*p+d*d*p+d*d*d*p;

		DbUnitAddress auxUnit=new DbUnitAddress();
		DbUnitAddress indexUnit=new DbUnitAddress();

		if (first)
		    {
			LobPrimitiveSsdDescriptorDriver.getContentExtension(dbsm,transactionNumber,id,indexUnit);
			if (indexUnit.isNull())
			    {
				LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,indexUnit);
				LobPrimitiveSsdDescriptorDriver.setContentExtension(dbsm,transactionNumber,
										    id,indexUnit);
			    }
			first=false;
		    }
		else
		    {
			auxUnit.copyFrom(indexUnit);
			LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,auxUnit,(int)(d-1),indexUnit);
			if (indexUnit.isNull())
			    {
				LobAddressesUnitDriver.createAddressesUnit(dbsm,transactionNumber,indexUnit);
				LobAddressesUnitDriver.setAddress(dbsm,transactionNumber,
								  auxUnit,(int)(d-1),indexUnit);
			    }
		    }
		if (start<lim)
		    {
			l=writeInIndexUnit(dbsm,transactionNumber,indexUnit,start-prevLim,(int)len,b,offset);
			start+=l;
			offset+=l;
			len-=l;
			r+=l;
		    }
		else
		    fillIndexUnit(dbsm,transactionNumber,indexUnit);
	    }

	if (from+r>size)
	    LobPrimitiveSsdDescriptorDriver.setContentSize(dbsm,transactionNumber,id,from+r);
	return r;
    }

    
    // Reading methods.

    static private int readInDataUnit(DbStorageManager dbsm, long transactionNumber,
				      DbUnitAddress dataUnit, long from, int length, byte[] b, int off)
	throws StorageException, IOException
    {
	return LobContentUnitDriver.getContent(dbsm,transactionNumber,dataUnit,(int)from,length,b,off);
    }

    static private int readInIndirectUnit(DbStorageManager dbsm, long transactionNumber,
					 DbUnitAddress indirectUnit, long from, int length, byte[] b, int off)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long addressNumber=from/p;
	DbUnitAddress dataUnit=new DbUnitAddress();
	
	long start=from;
	int offset=off;
	long len=p*d-from;
	if (len>length)
	    len=length;
	int l=0;
	int r=0;
	while (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indirectUnit,
						  (int)addressNumber,dataUnit);
		l=readInDataUnit(dbsm,transactionNumber,dataUnit,start%p,(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
		addressNumber++;
	    }
	return r;
    }


    static private int readInDoubleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
					       DbUnitAddress doubleIndirectUnit, long from, int length, 
					       byte[] b, int off)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long addressNumber=from/(d*p);
	DbUnitAddress indirectUnit=new DbUnitAddress();

	long start=from;
	int offset=off;
	long len=p*d*d-from;
	if (len>length)
	    len=length;
	int l=0;
	int r=0;
	while (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,doubleIndirectUnit,
						  (int)addressNumber,indirectUnit);
		l=readInIndirectUnit(dbsm,transactionNumber,indirectUnit,start%(p*d),(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
		addressNumber++;
	    }
	return r;
    }


    static private int readInTripleIndirectUnit(DbStorageManager dbsm, long transactionNumber,
					       DbUnitAddress tripleIndirectUnit, long from, int length, 
					       byte[] b, int off)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long addressNumber=from/(d*d*p);
	DbUnitAddress doubleIndirectUnit=new DbUnitAddress();

	long start=from;
	int offset=off;
	long len=p*d*d*d-from;
	if (len>length)
	    len=length;
	int l=0;
	int r=0;
	while (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,tripleIndirectUnit,
						  (int)addressNumber,doubleIndirectUnit);
		l=readInDoubleIndirectUnit(dbsm,transactionNumber,doubleIndirectUnit,
					    start%(p*d*d),(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
		addressNumber++;
	    }
	return r;
    }

    static private int readInIndexUnit(DbStorageManager dbsm, long transactionNumber,
				      DbUnitAddress indexUnit, long from, int length, byte[] b, int off)
	throws StorageException, IOException
    {
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;
	long lim1=(d-4)*p;
	long lim2=lim1+d*p;
	long lim3=lim2+d*d*p;
	long lim4=lim3+d*d*d*p;

	DbUnitAddress auxUnit=new DbUnitAddress();

	long start=from;
	int offset=off;
	long len=lim4-from;
	if (len>length)
	    len=length;
	int l=0;
	int r=0;
	for (long i=0; len>0 && i<d-4; i++)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)i,auxUnit);
		if (start>=i*p && start<(i+1)*p)
		    {
			l=readInDataUnit(dbsm,transactionNumber,auxUnit,start%p,(int)len,b,offset);
			start+=l;
			offset+=l;
			len-=l;
			r+=l;
		    }
	    }
	if (len>0 && start<lim2)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-4),auxUnit);
		l=readInIndirectUnit(dbsm,transactionNumber,auxUnit,start-lim1,(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
	    }
	if (len>0 && start<lim3)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-3),auxUnit);
		l=readInDoubleIndirectUnit(dbsm,transactionNumber,auxUnit,start-lim2,(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
	    }
	if (len>0)
	    {
		LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,indexUnit,(int)(d-2),auxUnit);
		l=readInTripleIndirectUnit(dbsm,transactionNumber,auxUnit,start-lim3,(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
	    }
	return r;
    }

    /**
     * Reads from a LOB datum and stores the readed in a byte array.
     * @param dbsm The data base storage manager where the operation will be performed.
     * @param transactionNumber The number of transaction.
     * @param id The identifier of the datum to read.
     * @param from The position of the LOB from which the reading starts.
     * @param maxLength The maximum number of bytes to read.
     * @param b The array where the readed bytes will be stored.
     * @param off The offset of <code>b</code> from which the readed bytes are stored.
     * @return The number of readed bytes. 
     */
    static public int read(DbStorageManager dbsm, long transactionNumber,
			   PhysicalID id, long from, int maxLength, byte[] b, int off)
	throws StorageException, IOException, OutOfLOBException
    {
	long size=LobPrimitiveSsdDescriptorDriver.getContentSize(dbsm,transactionNumber,id);
	if (from<0 || !(from+maxLength>=0 && from+maxLength<=MAX_LOB_SIZE && from<size))
	    throw new OutOfLOBException();
	    
	long p=dbsm.getPageSize();
	long d=p/DbUnitAddress.SIZE;

	long start=from;
	int offset=off;
	long len=size-from;
	if (len>maxLength)
	    len=maxLength>0?maxLength:0;
	int l=0;
	int r=0;

	long lim=dbsm.ID_SEGMENT_UNIT_SIZE-LobPrimitiveSsdDescriptorDriver.CONTENT_OFFSET;

	if (start<lim)
	    {
		l=LobPrimitiveSsdDescriptorDriver.getContent(dbsm,transactionNumber,id,(int)start,(int)len,b,offset);
		start+=l;
		offset+=l;
		len-=l;
		r+=l;
	    }
	boolean first=true;
	long prevLim=0;
	while (len>0)
	    {
		prevLim=lim;
		lim+=(d-4)*p+d*p+d*d*p+d*d*d*p;

		DbUnitAddress auxUnit=new DbUnitAddress();
		DbUnitAddress indexUnit=new DbUnitAddress();

		if (first)
		    {
			LobPrimitiveSsdDescriptorDriver.getContentExtension(dbsm,transactionNumber,id,indexUnit);
			first=false;
		    }
		else
		    {
			auxUnit.copyFrom(indexUnit);
			LobAddressesUnitDriver.getAddress(dbsm,transactionNumber,auxUnit,(int)(d-1),indexUnit);
		    }
		if (start<lim)
		    {
			l=readInIndexUnit(dbsm,transactionNumber,indexUnit,start-prevLim,(int)len,b,offset);
			start+=l;
			offset+=l;
			len-=l;
			r+=l;
		    }
	    }

	return r;
    }
}
