package DbStorage.manager;

import DbStorage.*;
import ssdbms.ssd.LabeledSSD;
import ssdbms.ssd.Label;
import ssdbms.ssd.SSDException;

/**
 * This class represents a pair formed of a <code>PhysicalID</code> and a string label
 */
public final class PhysicalLabeledSSD
{
    private String label;
    private PhysicalID id;

    /**
     * Constructs <code>PhysicalLabeledSSD</code> with null components.
     */
    public PhysicalLabeledSSD()
    {
	this.label=null;
	this.id=null;
    }

    /**
     * Constructs <code>PhysicalLabeledSSD</code> with specified components.
     */
    public PhysicalLabeledSSD(String label, PhysicalID id)
    {
	this.label=label;
	this.id=id;
    }

    /**
     * Returns a <code>LabeledSSD</code> from this.
     */
    public LabeledSSD getLabeledSSD()
    {
	try {
	    if (id!=null && !id.isNull())
		return new LabeledSSD(new Label(label),id);
	    else
		return null;
	} catch (SSDException e) {
	    return null;
	}
    }

    /**
     * Returns the label component.
     */
    public String getLabel()
    {
	return label;
    }

    /**
     * Returns the ID component.
     */
    public PhysicalID getID()
    {
	return id;
    }

    /**
     * Sets the label component.
     */
    public void setLabel(String label)
    {
	this.label=label;
    }

    /**
     * Sets the ID component.
     */
    public void setID(PhysicalID id)
    {
	this.id=id;
    }

    /**
     * This class is used to compare two <code>PhysicalLabeledSSD</code>s
     * using their ID component first and their label component second. 
     */
    static public class ByIDComparator
    {
	/**
	 * Compares two <code>PhysicalLabeledSSD</code>s
	 * using their ID component first and their label component second. 
	 */
	static public int compare(PhysicalLabeledSSD ls1, PhysicalLabeledSSD ls2)
	{
	    int r1=ls1.id.compareTo(ls2.id);
	    if (r1==0)
		{
		    int r2=ls1.label.compareTo(ls2.label);
		    if (r2<0)
			return -1;
		    else if (r2>0)
			return 1;
		    else
			return 0;
		}
	    else if (r1<0)
		return -1;
	    else
		return 1;
	}
    }

    /**
     * This class is used to compare two <code>PhysicalLabeledSSD</code>s
     * using their label component first and their ID component second. 
     */
    static public class ByLabelComparator
    {
	/**
	 * Compares two <code>PhysicalLabeledSSD</code>s
	 * using their label component first and their ID component second. 
	 */
	static public int compare(PhysicalLabeledSSD ls1, PhysicalLabeledSSD ls2)
	{
	    int r1=ls1.label.compareTo(ls2.label);
	    if (r1==0)
		{
		    int r2=ls1.id.compareTo(ls2.id);
		    if (r2<0)
			return -1;
		    else if (r2>0)
			return 1;
		    else
			return 0;
		}
	    else if (r1<0)
		return -1;
	    else
		return 1;
	}
    }
}
