package concurrency;

/**
 * This class represents the locking modes (share or exclusive).
 */
public class LockMode
{
    /** The byte representation of exclusive mode. */
    static public final byte EXCLUSIVE=(byte)1;

    /** The byte representation of share mode. */
    static public final byte SHARE=(byte)0;

    /** The exclusive mode. */
    static public final LockMode EXCLUSIVE_MODE=new LockMode(EXCLUSIVE);
    
    /** The share mode. */
    static public final LockMode SHARE_MODE=new LockMode(SHARE);
    
    byte lockMode;

    /**
     * Constructs a new locking mode with a specified value.
     */
    public LockMode(byte lockMode)
    {
	if (lockMode==EXCLUSIVE)
	    this.lockMode=EXCLUSIVE;
	else
	    this.lockMode=SHARE;
    }

    /**
     * Constructs a new locking mode as a copy of other.
     */
    public LockMode(LockMode lockMode)
    {
	this.lockMode=lockMode.lockMode;
    }

    /**
     * Constructa a new locking mode (share mode);
     */
    public LockMode()
    {
	this.lockMode=SHARE;
    }

    /**
     * Returns the byte representation of the locking mode.
     */
    public byte getMode()
    {
	return lockMode;
    }

    /**
     * Checks if the mode is exclusive.
     */
    public boolean isExclusive()
    {
	return lockMode==EXCLUSIVE;
    }

    /**
     * Checks if the mode is share.
     */
    public boolean isShare()
    {
	return lockMode==SHARE;
    }

    /**
     * Changes the mode.
     */
    public void setMode(byte lockMode)
    {
	if (lockMode==EXCLUSIVE)
	    this.lockMode=EXCLUSIVE;
	else
	    this.lockMode=SHARE;
    }

    /**
     * Try to change the mode preserving the greater (exclusive is greater than share).
     */
    public void setMorePowerful(byte lockMode)
    {
	if (lockMode==SHARE)
	    this.lockMode=lockMode;
    }

    /**
     * Try to change the mode preserving the greater (exclusive is greater than share).
     */
    public void setMorePowerful(LockMode lockMode)
    {
	setMorePowerful(lockMode.lockMode);
    }
    
    /**
     * Changes the mode to exclusive.
     */
    public void setExclusive()
    {
	lockMode=EXCLUSIVE;
    }

    /**
     * Changes the mode to share.
     */
    public void setShare()
    {
	lockMode=SHARE;
    }
    
    /**
     * Checks if this lock is compatible with oter.
     */
    public boolean isCompatible(LockMode mode)
    {
	if (lockMode==SHARE && mode.lockMode==SHARE)
	    return true;
	else
	    return false;
    }

    public String toString()
    {
	if (lockMode==EXCLUSIVE)
	    return "EXCLUSIVE";
	else
	    return "SHARE";
    }
}
