package ssdbms.manager.primitives;

import ssdbms.ssd.BLOBValue;
import java.io.*;

/**
 * Manages the BLOB operations.
 *
 * @see ssdbms.ssd.BLOBValue
 * @author Egar Arturo Garca Crdenas
 */
public interface BLOBManager extends BLOBValue
{
    /**
     * Reads the BLOB from a given position.
     *
     * @param pos The position of the BLOB from which it begins to read.
     * @param bytes Where the readed content will be storaged.
     * @return The number of readed bytes.
     */
    public int read(long pos, byte[] bytes)
	throws  OutOfBoundsException, BadFunctionException, FatalErrorException;

    /**
     * Set part of a byte array on the BLOB from a given position, if there exist 
     * previous bytes in a matching positions they will be overwrote, in 
     * other cases the will be appended.
     *
     * @param pos The position of the BLOB from which it begins to read.
     * @param bytes Where the readed content will be storaged.
     * @param offset The start offset of <code>bytes</code>. 
     * @param length The maximum number of bytes to read.
     * @return The number of readed bytes.
     */
    public int read(long pos, byte[] bytes, int offset, int length)
	throws  OutOfBoundsException, BadFunctionException, FatalErrorException;
    
    /**
     * Writes in the BLOB from a given position.
     *
     * @param pos The position of the BLOB from which it begins to write.
     * @param bytes Where the content to write is storaged.
     */
    public int write(long pos, byte[] bytes)
	throws  OutOfBoundsException, BadFunctionException, FatalErrorException;
    
    /**
     * Writes in the BLOB from a given position.
     *
     * @param pos The position of the BLOB from which it begins to write.
     * @param bytes Where the content to write is storaged.
     * @param offset The start offset of <code>bytes</code>. 
     * @param length The maximum number of bytes to write.
     */
    public int write(long pos, byte[] bytes, int offset, int length)
	throws  OutOfBoundsException, BadFunctionException, FatalErrorException;
    
    /**
     * Resizes the BLOB to the given length.
     *
     * @param length The new length of the BLOB in bytes.
     */
    public void setLength(long length)
	throws  OutOfBoundsException, BadFunctionException, FatalErrorException;
    
    /**
     * Returns the lenght in bytes of the storaged object.
     *
     * @return The lenght of the BLOB in bytes.
     */
    public long length()
	throws BadFunctionException, FatalErrorException;
}
