package ssdbms.manager.primitives;

import ssdbms.ssd.CLOBValue;
import java.io.*;

/**
 * Manages the CLOB operations.
 *
 * @see ssdbms.ssd.CLOBValue
 * @author Egar Arturo Garca Crdenas
 */
public interface CLOBManager extends CLOBValue
{
    /**
     * Reads the CLOB from a given position.
     *
     * @param pos The position of the CLOB from which it begins to read.
     * @param size The size of the string to be readed.
     * @return The readed string.
     */
    public String read(long pos, int size)
	 throws OutOfBoundsException, BadFunctionException, FatalErrorException;

    /**
     * Writes a string in the CLOB from a given position.
     *
     * @param pos The position of the CLOB from which it begins to write.
     * @param content The string to write.
     */
    public int write(long pos, String content)
	throws OutOfBoundsException, BadFunctionException, FatalErrorException;
    
    /**
     * Writes in the CLOB from a given position.
     *
     * @param pos The position of the CLOB from which it begins to write.
     * @param content The string contains the substring to write in the CLOB.
     * @param offset The start offset of <code>content</code>. 
     * @param length The maximum number of characters to write.
     */
    public int write(long pos, String content, int offset, int length)
	throws OutOfBoundsException, BadFunctionException, FatalErrorException;
    
    /**
     * Resizes the CLOB to the given length (in characters).
     *
     * @param length The new length of the CLOB.
     */
    public void setLength(long length)
	throws OutOfBoundsException, BadFunctionException, FatalErrorException;


    /**
     * Returns the lenght in characters of the storaged object.
     *
     * @return The lenght of the CLOB in characters.
     */
    public long length()
	throws BadFunctionException, FatalErrorException;
}
