package ssdbms.manager.primitives;

import ssdbms.ssd.*;

/**
 * A <code>SetOfID</code> is used to storage labeled semiestructured data (pairs label-identifier),
 * and recover them in sequential order.
 *
 * @see LabeledSSD
 * @author Egar Arturo Garca Crdenas
 */
public interface SetOfLabeledSSD extends NonPrimitiveValue
{
    /**
     * Returns <code>true</code> if the sequence has more elements. 
     * It means, the sequence has not been completely covered.
     *
     * @return <code>true</code> if there is more labeled semistructured data, <code>false</code> otherwise. 
     */ 
    public boolean hasNext() throws BadFunctionException, FatalErrorException;
    
    /**
     * Returns the next element in the sequence and shift one element.
     *
     * @return The next labeled semistructured data in the sequence, 
     * the first time that this method
     * is called return the firs element in the sequence,  
     * <code>null</code> if there is not more elements.
     */ 
    public LabeledSSD next() throws BadFunctionException, FatalErrorException;
}





