package ssdbms.ssd;

/**
 * This interface represents the identifiers of semistructured data.
 * A semiestructured datum is formed by an identifier and a value.
 * The identifiers used by a particular data manager must implement this interface.
 *
 * @author Egar Arturo Garca Crdenas
 */
public interface ID
{
    /**
     * Indicates if a identifier is valid or not. 
     * Invalid identifiers could be used in the same way that the <code>null</code> value 
     *  to represent an inexisting datum.
     *  
     *  @return <code>true</code> if the identifier is valid, <code>false</code> if is invalid.
     */
    public boolean isNID();

    /**
     * Indicates wheter some identifier is 'equal to' this one.
     *
     * @param id The identifier with which to compare.
     * @return <code>true</code> if <code>id</code> represents the same identifier that this one, 
     * <code>false</code> otherwise.
     */  
    public boolean equals(ID id);

    /**
     * Returns a string representation of the identifier;
     */
    public String toString();
}
