package ssdbms.ssd;

import java.util.regex.Pattern;

/**
 * This class represents a label for a semistructured datum contained in other.
 * <code>Label</code> class warp <code>Strigs<code> to restrict them, a label only
 * contains word characters i.e. a to z, A to Z, 0 to 9 or _. 
 *
 * @author Egar Arturo Garca Crdenas
 */
public final class Label
{
    static private Pattern labelPattern = Pattern.compile("\\w+");

    private String text;

    /**
     * Creates a new label from a given <code>String</code>.
     *
     * @param text The text to be used for the label.
     * @exception BadLabelException If <code>text</code> contains non word characters.
     */
    public Label(String text) throws BadLabelException
    {
	if (text!=null && labelPattern.matcher(text).matches())
	    this.text=text;
	else
	    throw new BadLabelException();
    } 

    /**
     * Return the text of the label.
     *
     * @return The text of the label.
     */
    public String getText()
    {
	return this.text;
    }

    /**
     * Indicates wheter some label is 'equal to' this one.
     * Two labes are equals if their text is equal.
     *
     * @param label The label with which to compare.
     * @return <code>true</code> if <code>x</code> has equal text that this one, 
     * <code>false</code> otherwise.
     */  
    public int compareTo(Label label)
    {
	return this.text.compareTo(label.text);
    }

    /**
     * Indicates wheter some label is 'equal to' this one.
     *
     * @param label The label with which to compare.
     * @return <code>true</code> if <code>label</code> represents the same label that this one, 
     * <code>false</code> otherwise.
     */  
    public boolean equals(Label label)
    {
	return text.equals(label.text);
    }

    public boolean equals(Object obj)
    {
	if (obj instanceof Label)
	    return equals((Label)obj);
	else
	    return false;
    }

    public int hashCode()
    {
	return text.hashCode();
    }

    /**
     * Return a string representation of the label.
     */
    public String toString()
    {
	return this.getText();
    }
}
