package ssdbms.ssd;

/**
 * <code>LabeledSSD</code> class represents labeled semistructured data
 * which actually are label-identifier pairs. 
 * Non primitive semistructured data contain label-identifier pairs.
  *
  * @see Label
  * @see ID
  * @author Egar Arturo Garca Crdenas
*/
public final class LabeledSSD
{
    private Label label;
    private ID id;

    /**
     * Creates a label-identifier pair known as labeled semistructured datum. 
     *
     * @param label The label component of the pair.
     * @param id The identifier component of the pair.
     * @exception SSDException If <code>label</code> or <code>id</code> are <code>null</code>
     */
    public LabeledSSD(Label label, ID id) throws SSDException
    {
	if (label==null || id==null)
	    throw new SSDException("A LabeledSSD can't be initialized from null values");
	else
	    {
		this.label=label;
		this.id=id;
	    }
    }
    
    /**
     *  Returs the identifier component of a labeled semistructured datum.
     *
     * @return The identifier component of the label-identifier pair.
     */
    public ID getID()
    {
	return id;
    }

    /**
     *  Returs the label component of a labeled semistructured datum.
     *
     * @return The label component of the label-identifier pair.
     */
    public Label getLabel()
    {
	return label;
    }

    /**
     * Indicates wheter some labeled semistructured data is 'equal to' this one.
     *
     * @param labeledSSD The label semistructured data with which to compare.
     * @return <code>true</code> if both have the same label component and the same identifier component, 
     * <code>false</code> otherwise.
     */  
    public boolean equals(LabeledSSD labeledSSD)
    {
	return label.equals(labeledSSD.getLabel()) && id.equals(labeledSSD.getID());
    }

    /**
     * Return a string representation of the labeled ssd.
     */
    public String toString()
    {
	return label.toString()+":"+id.toString();
    }
}
