package ssdbms.ssd;

/**
 * This interface represents the value of a semistructured datum.
 * A semiestructured datum is formed by an identifier and a value.
 * Subclasses of this class represent the different kinds of values,
 * basicaly there are two kinds of values: primitives and non primitives.
 *
 * @see PrimitiveValue
 * @see NonPrimitiveValue
 * @author Egar Arturo Garca Crdenas
 */
public interface SSDValue
{
    /**
     * Used to indicate a non primitive value.
     * @see NonPrimitiveValue
     */
    static public final int NON_PRIMITIVE=0;  
    
    /**
     * Used to indicate a primitive value of integer type.
     * @see IntegerValue
    */
    static public final int INTEGER=1;

    /**
     * Used to indicate a primitive value of real (float) type.
     * @see RealValue
     */
    static public final int REAL=2;

    /**
     * Used to indicate a primitive value of string type.
     * @see StringValue
     */
    static public final int STRING=3;

    
    /**
     * Used to indicate a primitive value of byte sequence type.
     * @see ByteSequenceValue
     */
    static public final int BYTE_SEQUENCE=4;

    
    /**
     * Used to indicate a primitive value of BLOB (Binary Large Object) type.
     * @see BLOBValue
     */
    static public final int BLOB=5;
    

    /**
     * Used to indicate a primitive value of CLOB (Character Large Object) type.
     * @see CLOBValue
     */
    static public final int CLOB=6;
}
